@@ -10,29 +10,25 @@ import numpy as np
 from typing import Union, Optional, Sequence, Dict, Tuple
 from instancelib import InstanceProvider
 
+from text_explainability.generation.surrogate import TreeSurrogate, RuleSurrogate
 
-class FeatureList:
+
+class BaseReturnType:
     def __init__(self,
                  used_features: Union[Sequence[str], Sequence[int]],
-                 scores: Union[Sequence[int], Sequence[float]],
                  labels: Optional[Sequence[int]] = None,
                  labelset: Optional[Sequence[str]] = None):
-        """Save scores per feature, grouped per label.
-
-        Examples of scores are feature importance scores, or counts of features in a dataset.
+        """Base return type.
 
         Args:
             used_features (Union[Sequence[str], Sequence[int]]): Used features per label.
-            scores (Union[Sequence[int], Sequence[float]]): Scores per label.
             labels (Optional[Sequence[int]], optional): Label indices to include, if none provided 
                 defaults to 'all'. Defaults to None.
             labelset (Optional[Sequence[str]], optional): Lookup for label names. Defaults to None.
         """
         self._used_features = used_features
-        self._labelset = labelset
-        print('LABELS::init', labels)
         self._labels = labels
-        self._scores = scores
+        self._labelset = labelset
 
     @property
     def labels(self):
@@ -68,6 +64,33 @@ class FeatureList:
             return self.labelset[idx]
         return idx
 
+    def __repr__(self) -> str:
+        labels = [self.label_by_index(label) for label in self.labels] if self.labels is not None else None
+        return f'{self.__class__.__name__}(labels={labels}, used_features={self.used_features})'
+
+
+class FeatureList(BaseReturnType):
+    def __init__(self,
+                 used_features: Union[Sequence[str], Sequence[int]],
+                 scores: Union[Sequence[int], Sequence[float]],
+                 labels: Optional[Sequence[int]] = None,
+                 labelset: Optional[Sequence[str]] = None):
+        """Save scores per feature, grouped per label.
+
+        Examples of scores are feature importance scores, or counts of features in a dataset.
+
+        Args:
+            used_features (Union[Sequence[str], Sequence[int]]): Used features per label.
+            scores (Union[Sequence[int], Sequence[float]]): Scores per label.
+            labels (Optional[Sequence[int]], optional): Label indices to include, if none provided 
+                defaults to 'all'. Defaults to None.
+            labelset (Optional[Sequence[str]], optional): Lookup for label names. Defaults to None.
+        """
+        super().__init__(used_features=used_features,
+                         labels=labels,
+                         labelset=labelset)
+        self._scores = scores
+
     def get_raw_scores(self, normalize: bool = False) -> np.ndarray:
         """Get saved scores per label as `np.ndarray`.
 
@@ -106,7 +129,6 @@ class FeatureList:
             return {self.label_by_index(label): [(feature, score_)
                     for feature, score_ in zip(self.used_features[label], all_scores[i])]
                     for i, label in enumerate(self.labels)}
-        print(self.labels)
         return {self.label_by_index(label): [(feature, score_)
                 for feature, score_ in zip(self.used_features, all_scores[i])]
                 for i, label in enumerate(self.labels)}
@@ -119,16 +141,65 @@ class FeatureList:
     def __str__(self) -> str:
         return '\n'.join([f'{a}: {str(b)}' for a, b in self.scores.items()])
 
-    def __repr__(self) -> str:
-        labels = [self.label_by_index(label) for label in self.labels] if self.labels is not None else None
-        return f'{self.__class__.__name__}(labels={labels}, used_features={self.used_features})'
 
+class DataExplanation:
+    def __init__(self,
+                 provider: InstanceProvider,
+                 sampled: bool = False):
+        """Save the sampled/generated instances used to determine an explanation.
 
-class FeatureAttribution(FeatureList):
+        Args:
+            provider (InstanceProvider): Sampled or generated data, including original instance.
+            sampled (bool, optional): Whether the data in the provider was sampled (True) or generated (False). 
+                Defaults to False.
+        """
+        self._provider = provider
+        self._original_instance = self._provider[next(iter(self._provider))]
+        self._neighborhood_instances = self._provider.get_children(self._original_instance)
+        self.sampled = sampled
+
+    @property
+    def original_instance(self):
+        """The instance for which the feature attribution scores were calculated."""
+        return self._original_instance
+
+    @property
+    def perturbed_instances(self):
+        """Perturbed versions of the original instance, if `sampled=False` during initialization."""
+        return None if self.sampled else self._neighborhood_instances
+
+    @property
+    def sampled_instances(self):
+        """Sampled instances, if `sampled=True` during initialization."""
+        return self._neighborhood_instances if self.sampled else None
+
+    @property
+    def neighborhood_instances(self):
+        """Instances in the neighborhood (either sampled or perturbed)."""
+        return self._neighborhood_instances
+
+
+class ReadableDataMixin:
+    @property
+    def used_features(self):
+        """Names of features of the original instance."""
+        if hasattr(self.original_instance, 'tokenized'):
+            return [self.original_instance.tokenized[i] for i in self._used_features]
+        return list(self._used_features)
+
+    def __repr__(self) -> str:
+        sampled_or_perturbed = 'sampled' if self.sampled else 'perturbed'
+        n = sum(1 for _ in self.neighborhood_instances)
+        labels = [self.label_by_index(label) for label in self.labels] if self.labels is not None else None
+        return f'{self.__class__.__name__}(labels={labels}, ' + \
+            f'used_features={self.used_features}, n_{sampled_or_perturbed}_instances={n})'
+
+
+class FeatureAttribution(ReadableDataMixin, FeatureList, DataExplanation):
     def __init__(self,
                  provider: InstanceProvider,
-                 used_features: Union[Sequence[str], Sequence[int]],
                  scores: Sequence[float],
+                 used_features: Optional[Union[Sequence[str], Sequence[int]]] = None,
                  scores_stddev: Sequence[float] = None,
                  base_score: float = None,
                  labels: Optional[Sequence[int]] = None,
@@ -141,8 +212,9 @@ class FeatureAttribution(FeatureList):
 
         Args:
             provider (InstanceProvider): Sampled or generated data, including original instance.
-            used_features (Union[Sequence[str], Sequence[int]]): Which features were selected for the explanation.
             scores (Sequence[float]): Scores corresponding to the selected features.
+            used_features (Optional[Union[Sequence[str], Sequence[int]]]): Selected features for the explanation label. 
+                Defaults to None.
             scores_stddev (Sequence[float], optional): Standard deviation of each feature attribution score. 
                 Defaults to None.
             base_score (float, optional): Base score, to which all scores are relative. Defaults to None.
@@ -151,52 +223,67 @@ class FeatureAttribution(FeatureList):
             sampled (bool, optional): Whether the data in the provider was sampled (True) or generated (False). 
                 Defaults to False.
         """
-        super().__init__(used_features=used_features,
-                         scores=scores,
-                         labels=labels,
-                         labelset=labelset)
-        self._provider = provider
+        DataExplanation.__init__(self,
+                                 provider=provider,
+                                 sampled=sampled)
+        if used_features is None:
+            used_features = list(range(len(self.original_instance.tokenized)))
+        FeatureList.__init__(self,
+                             used_features=used_features,
+                             scores=scores,
+                             labels=labels,
+                             labelset=labelset)
         self._base_score = base_score
         self._scores_stddev = scores_stddev
-        self._original_instance = self._provider[next(iter(self._provider))]
-        self._sampled_instances = self._provider.get_children(self._original_instance) if sampled else None
-        self._perturbed_instances = None if sampled else self._provider.get_children(self._original_instance) 
-
-    @property
-    def original_instance(self):
-        """The instance for which the feature attribution scores were calculated."""
-        return self._original_instance
-
-    @property
-    def perturbed_instances(self):
-        """Perturbed versions of the original instance, if `sampled=False` during initialization."""
-        return self._perturbed_instances
-
-    @property
-    def sampled_instances(self):
-        """Sampled instances, if `sampled=True` during initialization."""
-        return self._sampled_instances
-
-    @property
-    def neighborhood_instances(self):
-        """Instances in the neighborhood, either `sampled_instances` or `perturbed_instances`."""
-        return self.sampled_instances if self.sampled_instances is not None else self.perturbed_instances
-
-    @property
-    def used_features(self):
-        """Names of features of the original instance."""
-        if hasattr(self.original_instance, 'tokenized'):
-            return [self.original_instance.tokenized[i] for i in self._used_features]
-        return list(self._used_features)
 
     @property
     def scores(self):
         """Saved feature attribution scores."""
         return self.get_scores(normalize=False)
 
-    def __repr__(self) -> str:
-        sampled_or_perturbed = 'sampled' if self.sampled_instances is not None else 'perturbed'
-        n = sum(1 for _ in self.neighborhood_instances)
-        labels = [self.label_by_index(label) for label in self.labels] if self.labels is not None else None
-        return f'{self.__class__.__name__}(labels={labels}, ' + \
-            f'used_features={self.used_features}, n_{sampled_or_perturbed}_instances={n})'
+
+class Rules(ReadableDataMixin, BaseReturnType, DataExplanation):
+    def __init__(self,
+                 provider: InstanceProvider,
+                 rules: Union[Sequence[str], TreeSurrogate, RuleSurrogate],
+                 used_features: Optional[Union[Sequence[str], Sequence[int]]] = None,
+                 labels: Optional[Sequence[int]] = None,
+                 labelset: Optional[Sequence[str]] = None,
+                 sampled: bool = False):
+        """Base return type.
+
+        Args:
+            provider (InstanceProvider): Sampled or generated data, including original instance.
+            rules (Union[Sequence[str], TreeSurrogate, RuleSurrogate]): Rules applicable.
+            used_features (Optional[Union[Sequence[str], Sequence[int]]]): Used features per label. Defaults to None.
+            labels (Optional[Sequence[int]], optional): Label indices to include, if none provided 
+                defaults to 'all'. Defaults to None.
+            labelset (Optional[Sequence[str]], optional): Lookup for label names. Defaults to None.
+            sampled (bool, optional): Whether the data in the provider was sampled (True) or generated (False). 
+                Defaults to False.
+        """
+        DataExplanation.__init__(self,
+                                 provider=provider,
+                                 sampled=sampled)
+        if used_features is None:
+            used_features = list(range(len(self.original_instance.tokenized)))
+        BaseReturnType.__init__(self,
+                                used_features=used_features,
+                                labels=labels,
+                                labelset=labelset)
+        self._rules = self._extract_rules(rules)
+
+    def _extract_rules(self, rules: Union[Sequence[str], TreeSurrogate, RuleSurrogate]):
+        if isinstance(rules, (TreeSurrogate, RuleSurrogate)):
+            from skrules.rule import replace_feature_name
+            from skrules.skope_rules import BASE_FEATURE_NAME
+            feature_dict = {BASE_FEATURE_NAME + str(i): feat
+                            for i, feat in enumerate(self.used_features)}
+            return [(replace_feature_name(rule, feature_dict), perf)
+                     for rule, perf in rules.rules]
+        print(rules)
+        raise NotImplementedError('TODO: Support lists of rules')
+
+    @property
+    def rules(self):
+        return self._rules
